@@ -26,7 +26,6 @@ class MPU:
     self.excycles = 0
     self.addcycles = False
     self.processorCycles = 0
-    self.internalCycleDelay = 0
 
     if memory is None:
         memory = 0x10000 * [0x00]
@@ -68,7 +67,10 @@ class MPU:
 
   def WordAt(self, addr):
     return self.ByteAt(addr) + (self.ByteAt(addr + 1) << 8)
-  
+
+  def ProgramCounter(self):
+    return self.pc
+
   # Addressing modes
 
   def ImmediateByte(self):
@@ -169,24 +171,35 @@ class MPU:
     self.FlagsNZ(self.a)
 
   def opASL(self, x):
-    addr = x()
-    tbyte = self.ByteAt(addr)
+    if x is None:
+      tbyte = self.a
+    else:
+      addr = x()
+      tbyte = self.ByteAt(addr)
+
     self.flags &= ~(self.CARRY + self.NEGATIVE + self.ZERO)
 
     if tbyte & 128:
       self.flags |= self.CARRY
-      
     tbyte = (tbyte << 1) & 0xFF
+
     if tbyte:
       self.flags |= tbyte & 128
     else:
       self.flags |= self.ZERO
-
-    self.memory[addr] = tbyte
+    
+    if x is None:
+      self.a = tbyte
+    else:
+      self.memory[addr] = tbyte
 
   def opLSR(self, x):
-    addr = x()
-    tbyte = self.ByteAt(addr)
+    if x is None:
+      tbyte = self.a
+    else:
+      addr = x()
+      tbyte = self.ByteAt(addr)
+
     self.flags &=~(self.CARRY+self.NEGATIVE+self.ZERO)
     self.flags |=tbyte&1
 
@@ -195,7 +208,11 @@ class MPU:
       pass # {}
     else:
       self.flags |= self.ZERO
-    self.memory[addr]=tbyte
+
+    if x is None:
+      self.a = tbyte
+    else:
+      self.memory[addr]=tbyte
 
   def opBCL(self, x):
     if self.flags & x: 
@@ -207,7 +224,7 @@ class MPU:
     if self.flags & x:
       self.BranchRelAddr()
     else:
-      self.pc+=1
+      self.pc += 1
 
   def opCLR(self, x):
     self.flags &=~x
@@ -227,8 +244,12 @@ class MPU:
     self.flags |= tbyte&(128+64)
 
   def opROL(self, x):
-    addr = x()
-    tbyte = self.ByteAt(addr)
+    if x is None:
+      tbyte = self.a
+    else:
+      addr = x()
+      tbyte = self.ByteAt(addr)
+
     if self.flags & self.CARRY:
       if tbyte & 128:
         pass
@@ -241,14 +262,19 @@ class MPU:
       tbyte = tbyte << 1     
     tbyte &= 0xFF
     self.FlagsNZ(tbyte)
-    self.memory[addr] = tbyte
+
+    if x is None:
+      self.a = tbyte
+    else:
+      self.memory[addr] = tbyte
 
   def opEOR(self, x):
     self.a ^= self.ByteAt(x())
     self.FlagsNZ(self.a)
 
   def opADC(self, x):
-    data=self.ByteAt(x())
+    data = self.ByteAt(x())
+      
     if self.flags & self.DECIMAL:
       if self.flags & self.CARRY:
         tmp = 1
@@ -270,8 +296,11 @@ class MPU:
         tmp = 1
       else:
         tmp = 0
-      data += self.a + tmp
+      result = data + self.a + tmp
       self.flags &= ~(self.CARRY+self.OVERFLOW+self.NEGATIVE+self.ZERO)
+      if ( ~(self.a ^ data) & (self.a ^ result) ) & 0x80:
+        self.flags |= self.OVERFLOW
+      data = result
       if data > 255:
         self.flags|=self.OVERFLOW+self.CARRY
         data &=255
@@ -282,8 +311,12 @@ class MPU:
       self.a = data
 
   def opROR(self, x):
-    addr=x()
-    tbyte = self.ByteAt(addr)
+    if x is None:
+      tbyte = self.a
+    else:
+      addr=x()
+      tbyte = self.ByteAt(addr)
+
     if self.flags & self.CARRY:
       if tbyte & 1:
         pass # {}
@@ -295,7 +328,11 @@ class MPU:
         self.flags |= self.CARRY
       tbyte=tbyte>>1
     self.FlagsNZ(tbyte)
-    self.memory[addr]=tbyte
+
+    if x is None:
+      self.a = tbyte
+    else:
+      self.memory[addr] = tbyte
 
   def opSTA(self, x):
     self.memory[x()] = self.a
@@ -338,6 +375,7 @@ class MPU:
 
   def opSBC(self, x):
     data = self.ByteAt(x())
+      
     if self.flags & self.DECIMAL:
       if self.flags & self.CARRY:
         borrow = 0
@@ -372,26 +410,42 @@ class MPU:
       self.a = data & 0xFF
 
   def opDECR(self, x):
-    addr = x()
-    tbyte = self.ByteAt(addr)
+    if x is None:
+      tbyte = self.a
+    else:
+      addr = x()
+      tbyte = self.ByteAt(addr)
+
     self.flags &= ~(self.ZERO + self.NEGATIVE)
     tbyte = (tbyte - 1) & 0xFF
     if tbyte:
       self.flags |= tbyte & self.NEGATIVE
     else:
       self.flags |= self.ZERO
-    self.memory[addr] = tbyte
+
+    if x is None:
+      self.a = tbyte
+    else:
+      self.memory[addr] = tbyte
 
   def opINCR(self, x):
-    addr = x()
-    tbyte = self.ByteAt(addr)
+    if x is None:
+      tbyte = self.a
+    else:
+      addr = x()
+      tbyte = self.ByteAt(addr)
+
     self.flags &= ~(self.ZERO + self.NEGATIVE)
     tbyte = (tbyte + 1) & 0xFF
     if tbyte:
       self.flags |= tbyte & self.NEGATIVE
     else:
       self.flags |= self.ZERO
-    self.memory[addr] = tbyte
+
+    if x is None:
+      self.a = tbyte
+    else:
+      self.memory[addr] = tbyte
 
   def opLDA(self, x):
     self.a = self.ByteAt(x())
@@ -452,19 +506,12 @@ class MPU:
   
   @instruction(name="ORA", mode="imm", cycles=2)
   def inst_0x09(self):
-    self.a |= self.ImmediateByte()
-    self.FlagsNZ(self.a)
+    self.opORA(self.ProgramCounter)
     self.pc += 1
   
   @instruction(name="ASL", mode="acc", cycles=2)
   def inst_0x0a(self):
-    if self.a & 128:
-      self.flags |= self.CARRY
-    else:
-      self.flags &= ~self.CARRY
-    self.a = self.a << 1
-    self.FlagsNZ(self.a)
-    self.a &= 255
+    self.opASL(None)
 
   @instruction(name="ORA", mode="abs", cycles=4)
   def inst_0x0d(self):
@@ -545,32 +592,22 @@ class MPU:
   
   @instruction(name="AND", mode="imm", cycles=2)
   def inst_0x29(self):
-    self.a &= self.ImmediateByte()
-    self.FlagsNZ(self.a)
+    self.opAND(self.ProgramCounter)
     self.pc += 1
 
   @instruction(name="ROL", mode="acc", cycles=2)
   def inst_0x2a(self):
-    if self.flags & self.CARRY:
-      if (self.a & 128) == 0:
-        self.flags &=~self.CARRY
-      self.a = (self.a<<1) | 1
-    else:
-      if self.a & 128:
-        self.flags |= self.CARRY
-      self.a = self.a << 1
-    self.a &= 0xFF
-    self.FlagsNZ(self.a)
+    self.opROL(None)
 
   @instruction(name="BIT", mode="abs", cycles=4)
   def inst_0x2c(self):
     self.opBIT(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="AND", mode="abs", cycles=4)
   def inst_0x2d(self):
     self.opAND(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="ROL", mode="abs", cycles=6)
   def inst_0x2e(self):
@@ -603,7 +640,7 @@ class MPU:
   @instruction(name="AND", mode="aby", cycles=4, extracycles=1)
   def inst_0x39(self):
     self.opAND(self.AbsoluteYAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="AND", mode="abx", cycles=4, extracycles=1)
   def inst_0x3d(self):
@@ -613,7 +650,7 @@ class MPU:
   @instruction(name="ROL", mode="abx", cycles=7)
   def inst_0x3e(self):
     self.opROL(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="RTI", mode="imp", cycles=6)
   def inst_0x40(self):
@@ -623,17 +660,17 @@ class MPU:
   @instruction(name="EOR", mode="inx", cycles=6)
   def inst_0x41(self):
     self.opEOR(self.IndirectXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="EOR", mode="zpg", cycles=3)
   def inst_0x45(self):
     self.opEOR(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="LSR", mode="zpg", cycles=5)
   def inst_0x46(self):
     self.opLSR(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="PHA", mode="imp", cycles=3)
   def inst_0x48(self):
@@ -641,22 +678,12 @@ class MPU:
 
   @instruction(name="EOR", mode="imm", cycles=2)
   def inst_0x49(self):
-    self.a ^= self.ImmediateByte()
-    self.FlagsNZ(self.a)
-    self.pc+=1
+    self.opEOR(self.ProgramCounter)
+    self.pc += 1
 
   @instruction(name="LSR", mode="acc", cycles=2)
   def inst_0x4a(self):
-    self.flags &= ~(self.CARRY+self.NEGATIVE+self.ZERO)
-    if self.a & 1:
-      self.flags |= self.CARRY
-
-    self.a = self.a >> 1
-    if self.a:
-      pass # {}
-    else:
-      self.flags |= self.ZERO
-    self.a &= 255
+    self.opLSR(None)
 
   @instruction(name="JMP", mode="abs", cycles=3)
   def inst_0x4c(self):
@@ -665,7 +692,7 @@ class MPU:
   @instruction(name="EOR", mode="abs", cycles=4)
   def inst_0x4d(self):
     self.opEOR(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
   
   @instruction(name="LSR", mode="abs", cycles=6)
   def inst_0x4e(self):
@@ -679,17 +706,17 @@ class MPU:
   @instruction(name="EOR", mode="iny", cycles=5, extracycles=1)
   def inst_0x51(self):
     self.opEOR(self.IndirectYAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="EOR", mode="zpx", cycles=4)
   def inst_0x55(self):
     self.opEOR(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="LSR", mode="zpx", cycles=6)
   def inst_0x56(self):
     self.opLSR(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="CLI", mode="imp", cycles=2)
   def inst_0x58(self):
@@ -703,32 +730,32 @@ class MPU:
   @instruction(name="EOR", mode="abx", cycles=4, extracycles=1)
   def inst_0x5d(self):
     self.opEOR(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="LSR", mode="abx", cycles=7)
   def inst_0x5e(self):
     self.opLSR(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="RTS", mode="imp", cycles=6)
   def inst_0x60(self):
     self.pc=self.stPopWord()
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="ADC", mode="inx", cycles=6)
   def inst_0x61(self):
     self.opADC(self.IndirectXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="ADC", mode="zpg", cycles=3)
   def inst_0x65(self):
     self.opADC(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="ROR", mode="zpg", cycles=5)
   def inst_0x66(self):
     self.opROR(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="PLA", mode="imp", cycles=4)
   def inst_0x68(self):
@@ -737,53 +764,12 @@ class MPU:
 
   @instruction(name="ADC", mode="imm", cycles=2)
   def inst_0x69(self):
-    data = self.ImmediateByte()
-
-    if self.flags & self.CARRY:
-      tmp = 1
-    else:
-      tmp = 0
-
-    if self.flags & self.DECIMAL:
-      data = convert_to_bin(data) + convert_to_bin(self.a) + tmp
-      self.flags &= ~(self.CARRY+self.OVERFLOW+self.NEGATIVE+self.ZERO)
-      if data > 99:
-        self.flags |= self.CARRY+self.OVERFLOW
-        data -= 100
-      if data == 0:
-        self.flags |= self.ZERO
-      else:
-        self.flags |= self.data & 128
-      self.a = convert_to_bcd(data)
-    else:
-      if self.flags & self.CARRY:
-        tmp = 1
-      else:
-        tmp = 0
-      data += self.a + tmp
-      self.flags &= ~(self.CARRY+self.OVERFLOW+self.NEGATIVE+self.ZERO)
-      if data > 255:
-        self.flags |= self.OVERFLOW + self.CARRY
-        data &= 255
-      if data == 0:
-        self.flags |= self.ZERO
-      else:
-        self.flags |= data & 128
-      self.a=data
+    self.opADC(self.ProgramCounter)
     self.pc += 1
 
   @instruction(name="ROR", mode="acc", cycles=2)
   def inst_0x6a(self):
-    if self.flags & self.CARRY:
-      if (self.a & 1) == 0:
-        self.flags &= ~self.CARRY
-      self.a = (self.a >> 1) | 128
-    else:
-      if self.a & 1:
-        self.flags |= self.CARRY
-      self.a = self.a >> 1
-    self.FlagsNZ(self.a)
-    self.a &= 255
+    self.opROR(None)
 
   @instruction(name="JMP", mode="ind", cycles=5)
   def inst_0x6c(self):
@@ -798,7 +784,7 @@ class MPU:
   @instruction(name="ROR", mode="abs", cycles=6)
   def inst_0x6e(self):
     self.opROR(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
   
   @instruction(name="BVS", mode="rel", cycles=2, extracycles=2)
   def inst_0x70(self):
@@ -807,17 +793,17 @@ class MPU:
   @instruction(name="ADC", mode="iny", cycles=5, extracycles=1)
   def inst_0x71(self):
     self.opADC(self.IndirectYAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="ADC", mode="zpx", cycles=4)
   def inst_0x75(self):
     self.opADC(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="ROR", mode="zpx", cycles=6)
   def inst_0x76(self):
     self.opROR(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="SEI", mode="imp", cycles=2)
   def inst_0x78(self):
@@ -826,37 +812,37 @@ class MPU:
   @instruction(name="ADC", mode="aby", cycles=4, extracycles=1)
   def inst_0x79(self):
     self.opADC(self.AbsoluteYAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="ADC", mode="abx", cycles=4, extracycles=1)
   def inst_0x7d(self):
     self.opADC(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
   
   @instruction(name="ROR", mode="abx", cycles=7)
   def inst_0x7e(self):
     self.opROR(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="STA", mode="inx", cycles=6)
   def inst_0x81(self):
     self.opSTA(self.IndirectXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="STY", mode="zpg", cycles=3)
   def inst_0x84(self):
     self.opSTY(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="STA", mode="zpg", cycles=3)
   def inst_0x85(self):
     self.opSTA(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="STX", mode="zpg", cycles=3)
   def inst_0x86(self):
     self.opSTX(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="DEY", mode="imp", cycles=2)
   def inst_0x88(self):
@@ -872,17 +858,17 @@ class MPU:
   @instruction(name="STY", mode="abs", cycles=4)
   def inst_0x8c(self):
     self.opSTY(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
   
   @instruction(name="STA", mode="abs", cycles=4)
   def inst_0x8d(self):
     self.opSTA(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="STX", mode="abs", cycles=4)
   def inst_0x8e(self):
     self.opSTX(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
   
   @instruction(name="BCC", mode="rel", cycles=2, extracycles=2)
   def inst_0x90(self):
@@ -891,22 +877,22 @@ class MPU:
   @instruction(name="STA", mode="iny", cycles=6)
   def inst_0x91(self):
     self.opSTA(self.IndirectYAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="STY", mode="zpx", cycles=4)
   def inst_0x94(self):
     self.opSTY(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="STA", mode="zpx", cycles=4)
   def inst_0x95(self):
     self.opSTA(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="STX", mode="zpy", cycles=4)
   def inst_0x96(self):
     self.opSTX(self.ZeroPageYAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="TYA", mode="imp", cycles=2)
   def inst_0x98(self):
@@ -916,7 +902,7 @@ class MPU:
   @instruction(name="STA", mode="aby", cycles=5)
   def inst_0x99(self):
     self.opSTA(self.AbsoluteYAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="TXS", mode="imp", cycles=2)
   def inst_0x9a(self):
@@ -925,39 +911,37 @@ class MPU:
   @instruction(name="STA", mode="abx", cycles=5)
   def inst_0x9d(self):
     self.opSTA(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="LDY", mode="imm", cycles=2)
-  def inst_0xa0(self):
-    self.y=self.ImmediateByte()
-    self.FlagsNZ(self.y)
-    self.pc+=1
+  def inst_0xa0(self):               
+    self.opLDY(self.ProgramCounter)
+    self.pc += 1
 
   @instruction(name="LDA", mode="inx", cycles=6)
   def inst_0xa1(self):
     self.opLDA(self.IndirectXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="LDX", mode="imm", cycles=2)
   def inst_0xa2(self):
-    self.x=self.ImmediateByte()
-    self.FlagsNZ(self.x)
-    self.pc+=1
+    self.opLDX(self.ProgramCounter)
+    self.pc += 1
 
   @instruction(name="LDY", mode="zpg", cycles=3)
   def inst_0xa4(self):
     self.opLDY(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="LDA", mode="zpg", cycles=3)
   def inst_0xa5(self):
     self.opLDA(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="LDX", mode="zpg", cycles=3)
   def inst_0xa6(self):
     self.opLDX(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="TAY", mode="imp", cycles=2)
   def inst_0xa8(self):
@@ -965,9 +949,8 @@ class MPU:
     self.FlagsNZ(self.y)
 
   @instruction(name="LDA", mode="imm", cycles=2)
-  def inst_0xa9(self):
-    self.a = self.ImmediateByte()
-    self.FlagsNZ(self.a)
+  def inst_0xa9(self):              
+    self.opLDA(self.ProgramCounter)
     self.pc += 1
 
   @instruction(name="TAX", mode="imp", cycles=2)
@@ -997,22 +980,22 @@ class MPU:
   @instruction(name="LDA", mode="iny", cycles=5, extracycles=1)
   def inst_0xb1(self):
     self.opLDA(self.IndirectYAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="LDY", mode="zpx", cycles=4)
   def inst_0xb4(self):
     self.opLDY(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="LDA", mode="zpx", cycles=4)
   def inst_0xb5(self):
     self.opLDA(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="LDX", mode="zpy", cycles=4)
   def inst_0xb6(self):
     self.opLDX(self.ZeroPageYAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="CLV", mode="imp", cycles=2)
   def inst_0xb8(self):
@@ -1021,7 +1004,7 @@ class MPU:
   @instruction(name="LDA", mode="aby", cycles=4, extracycles=1)
   def inst_0xb9(self):
     self.opLDA(self.AbsoluteYAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="TSX", mode="imp", cycles=2)
   def inst_0xba(self):
@@ -1031,34 +1014,27 @@ class MPU:
   @instruction(name="LDY", mode="abx", cycles=4, extracycles=1)
   def inst_0xbc(self):
     self.opLDY(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="LDA", mode="abx", cycles=4, extracycles=1)
   def inst_0xbd(self):
     self.opLDA(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="LDX", mode="aby", cycles=4, extracycles=1)
   def inst_0xbe(self):
     self.opLDX(self.AbsoluteYAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="CPY", mode="imm", cycles=2)
   def inst_0xc0(self):
-    tbyte = self.ImmediateByte()
-    self.flags &= ~(self.CARRY+self.ZERO+self.NEGATIVE)
-    if self.y == tbyte:
-      self.flags |= self.CARRY+self.ZERO
-    elif self.y > tbyte:
-      self.flags |= self.CARRY
-    else:
-      self.flags |= self.NEGATIVE
+    self.opCPY(self.ProgramCounter)
     self.pc += 1
 
   @instruction(name="CMP", mode="inx", cycles=6)
   def inst_0xc1(self):
     self.opCMP(self.IndirectXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="CPY", mode="zpg", cycles=3)
   def inst_0xc4(self):
@@ -1082,15 +1058,8 @@ class MPU:
     self.FlagsNZ(self.y)
 
   @instruction(name="CMP", mode="imm", cycles=2)
-  def inst_0xc9(self):
-    tbyte = self.ImmediateByte()
-    self.flags &= ~(self.CARRY+self.ZERO+self.NEGATIVE)
-    if self.a == tbyte:
-      self.flags |= self.CARRY + self.ZERO
-    elif self.a > tbyte:
-      self.flags |= self.CARRY
-    else:
-      self.flags |= self.NEGATIVE
+  def inst_0xc9(self):              
+    self.opCMP(self.ProgramCounter)
     self.pc +=1
 
   @instruction(name="DEX", mode="imp", cycles=2)
@@ -1121,17 +1090,17 @@ class MPU:
   @instruction(name="CMP", mode="iny", cycles=5, extracycles=1)
   def inst_0xd1(self):
     self.opCMP(self.IndirectYAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="CMP", mode="zpx", cycles=4)
   def inst_0xd5(self):
     self.opCMP(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="DEC", mode="zpx", cycles=6)
   def inst_0xd6(self):
     self.opDECR(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="CLD", mode="imp", cycles=2)
   def inst_0xd8(self):
@@ -1140,49 +1109,42 @@ class MPU:
   @instruction(name="CMP", mode="aby", cycles=4, extracycles=1)
   def inst_0xd9(self):
     self.opCMP(self.AbsoluteYAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="CMP", mode="abx", cycles=4, extracycles=1)
   def inst_0xdd(self):
     self.opCMP(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="DEC", mode="abx", cycles=7)
   def inst_0xde(self):
     self.opDECR(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="CPX", mode="imm", cycles=2)
   def inst_0xe0(self):
-    tbyte = self.ImmediateByte()
-    self.flags &= ~(self.CARRY+self.ZERO+self.NEGATIVE)
-    if self.x == tbyte:
-      self.flags |= self.CARRY + self.ZERO
-    elif self.x > tbyte:
-      self.flags |= self.CARRY
-    else:
-      self.flags |= self.NEGATIVE
+    self.opCPX(self.ProgramCounter)
     self.pc += 1
 
   @instruction(name="SBC", mode="inx", cycles=6)
   def inst_0xe1(self):
     self.opSBC(self.IndirectXAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="CPX", mode="zpg", cycles=3)
   def inst_0xe4(self):
     self.opCPX(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="SBC", mode="zpg", cycles=3)
   def inst_0xe5(self):
     self.opSBC(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="INC", mode="zpg", cycles=5)
   def inst_0xe6(self):
     self.opINCR(self.ZeroPageAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="INX", mode="imp", cycles=2)
   def inst_0xe8(self):
@@ -1192,39 +1154,7 @@ class MPU:
   
   @instruction(name="SBC", mode="imm", cycles=2)
   def inst_0xe9(self):
-    data=self.ImmediateByte()
-
-    if self.flags & self.DECIMAL:
-      if self.flags & self.CARRY:
-        tmp = 0
-      else:
-        tmp = 1
-      data = convert_to_bin(self.a) - convert_to_bin(data) - tmp
-      self.flags &= ~(self.CARRY+self.ZERO+self.NEGATIVE+self.OVERFLOW)
-      if data == 0:
-        self.flags |= self.ZERO + self.CARRY
-      elif data > 0:
-        self.flags |= self.CARRY
-      else:
-        self.flags |= self.NEGATIVE
-        data +=100
-      self.a = convert_to_bcd(data)
-    else:
-      if self.flags & self.CARRY:
-        tmp = 0
-      else:
-        tmp = 1    
-      data = self.a - data - tmp
-      self.flags &= ~(self.CARRY+self.ZERO+self.OVERFLOW+self.NEGATIVE)
-      if data == 0:
-        self.flags |= self.ZERO + self.CARRY
-      elif data > 0:
-        self.flags |= self.CARRY
-      else:
-        self.flags |= self.OVERFLOW
-      data &= 255
-      self.flags |= data & 128
-      self.a = data
+    self.opSBC(self.ProgramCounter)
     self.pc += 1
 
   @instruction(name="NOP", mode="imp", cycles=2)
@@ -1234,17 +1164,17 @@ class MPU:
   @instruction(name="CPX", mode="abs", cycles=4)
   def inst_0xec(self):
     self.opCPX(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
   
   @instruction(name="SBC", mode="abs", cycles=4)
   def inst_0xed(self):
     self.opSBC(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="INC", mode="abs", cycles=6)
   def inst_0xee(self):
     self.opINCR(self.AbsoluteAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="BEQ", mode="rel", cycles=2, extracycles=2)
   def inst_0xf0(self):
@@ -1253,17 +1183,17 @@ class MPU:
   @instruction(name="SBC", mode="iny", cycles=5, extracycles=1)
   def inst_0xf1(self):
     self.opSBC(self.IndirectYAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="SBC", mode="zpx", cycles=4)
   def inst_0xf5(self):
     self.opSBC(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
 
   @instruction(name="INC", mode="zpx", cycles=6)
   def inst_0xf6(self):
     self.opINCR(self.ZeroPageXAddr)
-    self.pc+=1
+    self.pc += 1
   
   @instruction(name="SED", mode="imp", cycles=2)
   def inst_0xf8(self):
@@ -1272,15 +1202,15 @@ class MPU:
   @instruction(name="SBC", mode="aby", cycles=4, extracycles=1)
   def inst_0xf9(self):
     self.opSBC(self.AbsoluteYAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="SBC", mode="abx", cycles=4, extracycles=1)
   def inst_0xfd(self):
     self.opSBC(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
   @instruction(name="INC", mode="abx", cycles=7)  
   def inst_0xfe(self):
     self.opINCR(self.AbsoluteXAddr)
-    self.pc+=2
+    self.pc += 2
 
